@@ -9,6 +9,11 @@
 int contains_data(FILE *fp, unsigned long ulPosition,
 		  const void *pData, unsigned int uiLen);
 
+/* Reads data of length uiLen at position ulPositoin.
+   The file pointer will change when calling this function! */
+int read_data(FILE *fp, unsigned long ulPosition,
+	      void *pData, unsigned int uiLen);
+
 /* Writes a data pattern of length uiLen at position ulPositoin.
    The file pointer will change when calling this function! */
 int write_data(FILE *fp, unsigned long ulPosition,
