@@ -33,13 +33,15 @@ int is_fat_32_br(FILE *fp)
    /* A "file" is probably some kind of FAT32 boot record if it contains the
       magic chars 0x55, 0xAA at positions 0x1FE, 0x3FE and 0x5FE */
    unsigned char aucRef[] = {0x55, 0xAA};
-   unsigned char aucMagic[] = {'M','S','W','I','N','4','.','1'};
+   unsigned char aucMagic9x[] = {'M','S','W','I','N','4','.','1'};
+   unsigned char aucMagicNT[] = {'M','S','D','O','S','5','.','0'};
    int i;
 
    for(i=0 ; i<3 ; i++)
       if( ! contains_data(fp, 0x1FE + i*0x200, aucRef, sizeof(aucRef)))
 	 return 0;
-   if( ! contains_data(fp, 0x03, aucMagic, sizeof(aucMagic)))
+   if(( ! contains_data(fp, 0x03, aucMagic9x, sizeof(aucMagic9x))) &&
+      ( ! contains_data(fp, 0x03, aucMagicNT, sizeof(aucMagicNT))))
       return 0;
    return 1;
 } /* is_fat_32_br */
@@ -124,53 +126,101 @@ int write_fat_32_fd_br(FILE *fp, int bKeepLabel)
 	   write_data(fp, 0x3f0, br_fat32_0x3f0, sizeof(br_fat32_0x3f0)) );
 } /* write_fat_32_fd_br */
 
-int entire_fat_32_nt_br_matches(FILE *fp)
+int entire_fat_32_nt5_br_matches(FILE *fp)
 {
    #include "br_fat32_0x0.h"
-   #include "br_fat32nt_0x52.h"
-   #include "br_fat32nt_0x3f0.h"
-   #include "br_fat32nt_0x1800.h"
+   #include "br_fat32nt5_0x52.h"
+   #include "br_fat32nt5_0x3f0.h"
+   #include "br_fat32nt5_0x1800.h"
 
    return
       ( contains_data(fp, 0x0, br_fat32_0x0, sizeof(br_fat32_0x0)) &&
 	/* BIOS Parameter Block might differ between systems */
-	contains_data(fp, 0x52, br_fat32nt_0x52, sizeof(br_fat32nt_0x52)) &&
+	contains_data(fp, 0x52, br_fat32nt5_0x52, sizeof(br_fat32nt5_0x52)) &&
 	/* Cluster information might differ between systems */
-	contains_data(fp, 0x3f0, br_fat32nt_0x3f0, sizeof(br_fat32nt_0x3f0)) &&
-	contains_data(fp, 0x1800, br_fat32nt_0x1800, sizeof(br_fat32nt_0x1800))
+	contains_data(fp, 0x3f0, br_fat32nt5_0x3f0, sizeof(br_fat32nt5_0x3f0)) &&
+	contains_data(fp, 0x1800, br_fat32nt5_0x1800, sizeof(br_fat32nt5_0x1800))
 	 );
-} /* entire_fat_32_nt_br_matches */
+} /* entire_fat_32_nt5_br_matches */
 
-int write_fat_32_nt_br(FILE *fp, int bKeepLabel)
+int write_fat_32_nt5_br(FILE *fp, int bKeepLabel)
 {
    #include "label_11_char.h"
    #include "br_fat32_0x0.h"
-   #include "br_fat32nt_0x52.h"
-   #include "br_fat32nt_0x3f0.h"
-   #include "br_fat32nt_0x1800.h"
+   #include "br_fat32nt5_0x52.h"
+   #include "br_fat32nt5_0x3f0.h"
+   #include "br_fat32nt5_0x1800.h"
 
    if(bKeepLabel)
       return
 	 ( write_data(fp, 0x0, br_fat32_0x0, sizeof(br_fat32_0x0)) &&
 	   /* BIOS Parameter Block should not be overwritten */
-	   write_data(fp, 0x52, br_fat32nt_0x52, sizeof(br_fat32nt_0x52)) &&
+	   write_data(fp, 0x52, br_fat32nt5_0x52, sizeof(br_fat32nt5_0x52)) &&
    /* Cluster information is not overwritten, however, it would be OK
       to write 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff here. */
-	   write_data(fp, 0x3f0, br_fat32nt_0x3f0, sizeof(br_fat32nt_0x3f0)) &&
-	   write_data(fp, 0x1800, br_fat32nt_0x1800, sizeof(br_fat32nt_0x1800))
+	   write_data(fp, 0x3f0, br_fat32nt5_0x3f0, sizeof(br_fat32nt5_0x3f0)) &&
+	   write_data(fp, 0x1800, br_fat32nt5_0x1800, sizeof(br_fat32nt5_0x1800))
 	    );
    else
       return
 	 ( write_data(fp, 0x0, br_fat32_0x0, sizeof(br_fat32_0x0)) &&
 	   /* BIOS Parameter Block should not be overwritten */
 	   write_data(fp, 0x47, label_11_char, sizeof(label_11_char)) &&
-	   write_data(fp, 0x52, br_fat32nt_0x52, sizeof(br_fat32nt_0x52)) &&
+	   write_data(fp, 0x52, br_fat32nt5_0x52, sizeof(br_fat32nt5_0x52)) &&
    /* Cluster information is not overwritten, however, it would be OK
       to write 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff here. */
-	   write_data(fp, 0x3f0, br_fat32nt_0x3f0, sizeof(br_fat32nt_0x3f0)) &&
-	   write_data(fp, 0x1800, br_fat32nt_0x1800, sizeof(br_fat32nt_0x1800))
+	   write_data(fp, 0x3f0, br_fat32nt5_0x3f0, sizeof(br_fat32nt5_0x3f0)) &&
+	   write_data(fp, 0x1800, br_fat32nt5_0x1800, sizeof(br_fat32nt5_0x1800))
 	    );
-} /* write_fat_32_nt_br */
+} /* write_fat_32_nt5_br */
+
+int entire_fat_32_nt6_br_matches(FILE *fp)
+{
+   #include "br_fat32nt6_0x0.h"
+   #include "br_fat32nt6_0x52.h"
+   #include "br_fat32nt6_0x5fe.h"
+   #include "br_fat32nt6_0x1800.h"
+
+   return
+      ( contains_data(fp, 0x0, br_fat32nt6_0x0, sizeof(br_fat32nt6_0x0)) &&
+	/* BIOS Parameter Block might differ between systems */
+	contains_data(fp, 0x52, br_fat32nt6_0x52, sizeof(br_fat32nt6_0x52)) &&
+	/* Cluster information might differ between systems */
+	contains_data(fp, 0x5fe, br_fat32nt6_0x5fe, sizeof(br_fat32nt6_0x5fe)) &&
+	contains_data(fp, 0x1800, br_fat32nt6_0x1800, sizeof(br_fat32nt6_0x1800))
+	 );
+} /* entire_fat_32_nt6_br_matches */
+
+int write_fat_32_nt6_br(FILE *fp, int bKeepLabel)
+{
+   #include "label_11_char.h"
+   #include "br_fat32nt6_0x0.h"
+   #include "br_fat32nt6_0x52.h"
+   #include "br_fat32nt6_0x5fe.h"
+   #include "br_fat32nt6_0x1800.h"
+
+   if(bKeepLabel)
+      return
+	 ( write_data(fp, 0x0, br_fat32nt6_0x0, sizeof(br_fat32nt6_0x0)) &&
+	   /* BIOS Parameter Block should not be overwritten */
+	   write_data(fp, 0x52, br_fat32nt6_0x52, sizeof(br_fat32nt6_0x52)) &&
+   /* Cluster information is not overwritten, however, it would be OK
+      to write 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff here. */
+	   write_data(fp, 0x5fe, br_fat32nt6_0x5fe, sizeof(br_fat32nt6_0x5fe)) &&
+	   write_data(fp, 0x1800, br_fat32nt6_0x1800, sizeof(br_fat32nt6_0x1800))
+	    );
+   else
+      return
+	 ( write_data(fp, 0x0, br_fat32nt6_0x0, sizeof(br_fat32nt6_0x0)) &&
+	   /* BIOS Parameter Block should not be overwritten */
+	   write_data(fp, 0x47, label_11_char, sizeof(label_11_char)) &&
+	   write_data(fp, 0x52, br_fat32nt6_0x52, sizeof(br_fat32nt6_0x52)) &&
+   /* Cluster information is not overwritten, however, it would be OK
+      to write 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0xff here. */
+	   write_data(fp, 0x5fe, br_fat32nt6_0x5fe, sizeof(br_fat32nt6_0x5fe)) &&
+	   write_data(fp, 0x1800, br_fat32nt6_0x1800, sizeof(br_fat32nt6_0x1800))
+	    );
+} /* write_fat_32_nt6_br */
 
 int entire_fat_32_pe_br_matches(FILE *fp)
 {
